#include "Document.hpp"
#include "Document-odb.hpp"

#include <odb/core.hxx>
#include <odb/sqlite/database.hxx>

#include <stdexcept>


const DbObject::Id DbObject::ID_NEW = ~DbObject::Id(0);


Document::Ptr Document::load(odb::database& db) {
  odb::transaction t(db.begin());
  typedef odb::result<Document> result_t;
  result_t r(db.query<Document>());
  if (r.empty()) {
    throw std::runtime_error(QObject::tr("no document in db file").toStdString());
  } else {
    Ptr o = r.begin().load();
    t.commit();
    return o;
  }
}

void Document::save(odb::database& db) {
  // query, assert 1, erase old, persist new
  odb::transaction t(db.begin());
  typedef odb::result<Document> result;
  result r(db.query<Document>());
  if (!r.empty()) {
    // remove old
    result::iterator it = r.begin();
    db.erase(*it);
    // ensure that was only one
    if (++it != r.end()) {
      throw std::runtime_error(tr("more than one document in db file").toStdString());
    }
  }
  // store this
  db.persist<Document>(*this);
  t.commit();

}