#pragma once

#include <QtCore/QSharedPointer>
#include <QtCore/QWeakPointer>
#include <QtCore/QUrl>
#include <QtCore/QObject>

#include <odb/database.hxx>


#pragma db object abstract
class DbObject: public QObject {
  Q_OBJECT
public:
  typedef unsigned long long Id;
  static const Id ID_NEW;

  /**
   * Object id in database
   *
   * @note it's mutable to drop it to default value on otherwise const operation 'erase'
   */
  #pragma db id auto
  Id id;

protected:
  DbObject(): id(ID_NEW) {}
  virtual ~DbObject() {}
};


#pragma db object polymorphic
class Document : public DbObject {
  Q_OBJECT

public:
  typedef QSharedPointer<Document> Ptr;

  /** load document from specified url */
  static Ptr load(odb::database& db);
  /** save document to specified url */
  virtual void save(odb::database& db);


  QString baseDocumentData;

protected:
  friend class odb::access;
  /** instance is only via odb load */
  explicit Document() {}
};



#pragma db object polymorphic
class Workflow: public Document {
  Q_OBJECT

public:
  typedef QSharedPointer<Workflow> Ptr;

  QString workflowData;
private:
  friend class odb::access;
};


